//--------------------------------------------------------------------------------------- 
//  FILE:    X2Character_MutonM2
//  AUTHOR:  Amineri (Long War Studios)
//  PURPOSE: Builds basic MutonM2 (Centurion) character template
//--------------------------------------------------------------------------------------- 
class X2Character_MutonM2 extends X2Character config(LW_MutonM2);

struct AIJobInfo_Addition
{
	var Name JobName;						// Name of this job.
	var Name Preceding;						// Name of unit before current in the list, if any
	var Name Succeeding;					// Name of unit after current in the list, if any
	var int DefaultPosition;				// Default index to insert at if cannot find based on name
};

var config array<AIJobInfo_Addition> JobListingAdditions; // Definition of qualifications for each job for this new character

static function array<X2DataTemplate> CreateTemplates()
{
	local array<X2DataTemplate> Templates;
	
	Templates.AddItem(CreateTemplate_MutonM2());
	
	//AddAIJobs(); // TODO: figure out how to add AIJobs for MutonM2 -- AIJobManager is not available at this point
	
	return Templates;
}

static function X2CharacterTemplate CreateTemplate_MutonM2()
{
	local X2CharacterTemplate CharTemplate;
	local LootReference Loot;

	`CREATE_X2CHARACTER_TEMPLATE(CharTemplate, 'MutonM2');
	CharTemplate.CharacterGroupName = 'Muton';
	CharTemplate.DefaultLoadout='MutonM2_Loadout';
	CharTemplate.BehaviorClass=class'XGAIBehavior';
	CharTemplate.strPawnArchetypes.AddItem("LWMutonM2.ARC_GameUnit_MutonM2"); // TODO : Update to Muton M2 archetype when complete
	Loot.ForceLevel=0;
	Loot.LootTableName='Muton_BaseLoot';  // TODO : possibly create new LootTable entry for MutonM2
	CharTemplate.Loot.LootReferences.AddItem(Loot);

	// Timed Loot
	Loot.ForceLevel = 0;
	Loot.LootTableName = 'Muton_TimedLoot';  // TODO : possibly create new Timed LootTable entry for MutonM2
	CharTemplate.TimedLoot.LootReferences.AddItem(Loot);

	//CharTemplate.strMatineePackage = "CIN_Muton"; 
	CharTemplate.strMatineePackages.AddItem("CIN_Muton"); //update with new cinematic?

	CharTemplate.UnitSize = 1;
	// Traversal Rules -- same as base Muton
	CharTemplate.bCanUse_eTraversal_Normal = true;
	CharTemplate.bCanUse_eTraversal_ClimbOver = true;
	CharTemplate.bCanUse_eTraversal_ClimbOnto = true;
	CharTemplate.bCanUse_eTraversal_ClimbLadder = true;
	CharTemplate.bCanUse_eTraversal_DropDown = true;
	CharTemplate.bCanUse_eTraversal_Grapple = false;
	CharTemplate.bCanUse_eTraversal_Landing = true;
	CharTemplate.bCanUse_eTraversal_BreakWindow = true;
	CharTemplate.bCanUse_eTraversal_KickDoor = true;
	CharTemplate.bCanUse_eTraversal_JumpUp = false;
	CharTemplate.bCanUse_eTraversal_WallClimb = false;
	CharTemplate.bCanUse_eTraversal_BreakWall = false;
	CharTemplate.bAppearanceDefinesPawn = false;    
	CharTemplate.bCanTakeCover = true;

	CharTemplate.bIsAlien = true;
	CharTemplate.bIsAdvent = false;
	CharTemplate.bIsCivilian = false;
	CharTemplate.bIsPsionic = false;
	CharTemplate.bIsRobotic = false;
	CharTemplate.bIsSoldier = false;

	CharTemplate.bCanBeTerrorist = false;
	CharTemplate.bCanBeCriticallyWounded = false;
	CharTemplate.bIsAfraidOfFire = true;

	CharTemplate.bAllowSpawnFromATT = false;

	CharTemplate.Abilities.AddItem('CounterattackPreparation');
	CharTemplate.Abilities.AddItem('CounterattackDescription');
	CharTemplate.Abilities.AddItem('WarCry');
	CharTemplate.Abilities.AddItem('Beastmaster');
	//CharTemplate.Abilities.AddItem('BayonetCharge');  // moved to X2Item_MutonM2.CreateTemplate_MutonM2_MeleeAttack so that proper animation plays

	CharTemplate.AddTemplateAvailablility(CharTemplate.BITFIELD_GAMEAREA_Multiplayer); // Allow in MP!
	CharTemplate.MPPointValue = CharTemplate.XpKillscore * 10;
	CharTemplate.strBehaviorTree = "MutonM2Root";

	//probably keep cinematic intro, since we can't create a new one -- alternatively no cinematic intro for tier 2
	CharTemplate.SightedNarrativeMoments.AddItem(XComNarrativeMoment'X2NarrativeMoments.TACTICAL.AlienSitings.T_Central_AlienSightings_Muton');

	CharTemplate.strTargetIconImage = class'UIUtilities_Image'.const.TargetIcon_Alien;

	return CharTemplate;
}

static function AddAIJobs()
{
	local X2AIJobManager JobMgr;
	local AIJobInfo_Addition Addition;
	local int AdditionIndex;
	local AIJobInfo JobInfo;
	local name MyName;

	MyName = 'MutonM2';
	
	//retrieve AIJobManager
	JobMgr = `AIJOBMGR;

	//TODO : figure out how to insert additional characters into the X2AIJobManager.JobInfo.ValidChar array
	foreach default.JobListingAdditions(Addition)
	{
		JobInfo = JobMgr.GetJobListing(Addition.JobName);
		if(JobInfo.JobName == '') continue;
		
		if(Addition.Preceding != '')
		{
			AdditionIndex = JobInfo.ValidChar.Find(Addition.Preceding);
			if(AdditionIndex != INDEX_NONE)
			{
				// found preceding, insert addition after it
				JobInfo.ValidChar.InsertItem(AdditionIndex+1, MyName);
				continue;
			}
		}
		if(Addition.Succeeding != '')
		{
			AdditionIndex = JobInfo.ValidChar.Find(Addition.Succeeding);
			if(AdditionIndex != INDEX_NONE)
			{
				// found succeeding, insert addition before it
				JobInfo.ValidChar.InsertItem(AdditionIndex, MyName);
				continue;
			}
		}

		//default to default index value
		if (Addition.DefaultPosition < JobInfo.ValidChar.Length)
		if(Addition.DefaultPosition < 0)
		{
			JobInfo.ValidChar.InsertItem(0, MyName);
		}
		
		if (JobInfo.ValidChar.Length > 0)
		{
			if (Addition.DefaultPosition < JobInfo.ValidChar.Length)
			{
				JobInfo.ValidChar.InsertItem(Addition.DefaultPosition, MyName);
			} else {  //  if this tile is not configured, use the last configured tile					
				JobInfo.ValidChar.AddItem(MyName);
			}
		}
	}
}










